/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
#pragma once

#include <mutex>

template <typename T>
class Synchronized {
   public:
    explicit Synchronized(const T& value) : value_(value) {}

    T get() const {
        std::lock_guard<std::mutex> lock(mutex_);
        return value_;
    }

    T&& release() {
        std::lock_guard<std::mutex> lock(mutex_);
        return std::move(value_);
    }

    Synchronized& operator=(const T& value) {
        std::lock_guard<std::mutex> lock(mutex_);
        value_ = value;
        return *this;
    }

   private:
    T value_;
    mutable std::mutex mutex_;
};
